@@ -1,15 +1,12 @@
-import attr
+import warnings
+from collections.abc import MutableSequence
 from typing import Optional
 
-try:
-    from collections.abc import MutableSequence
-except ImportError:
-    from collections import MutableSequence
-
-import warnings
+import attr
 from fontTools.pens.pointPen import PointToSegmentPen
 
 from ufoLib2.objects.point import Point
+from ufoLib2.objects.misc import getBounds, getControlBounds
 
 
 @attr.s(slots=True)
@@ -49,6 +46,21 @@ class Contour(MutableSequence):
             return True
         return self.points[0].type == "move"
 
+    def move(self, delta):
+        for point in self.points:
+            point.move(delta)
+
+    def getBounds(self, layer=None):
+        return getBounds(self, layer)
+
+    @property
+    def bounds(self):
+        # also add a property getter like defcon's, since we can...
+        return self.getBounds()
+
+    def getControlBounds(self, layer=None):
+        return getControlBounds(self, layer)
+
     # -----------
     # Pen methods
     # -----------
