@@ -1,5 +1,7 @@
-import attr
 from typing import Optional, Union
+
+import attr
+
 from ufoLib2.objects.misc import AttrDictMixin
 
 
@@ -18,12 +20,8 @@ class Guideline(AttrDictMixin):
             raise ValueError("x or y must be present")
         if x is None or y is None:
             if angle is not None:
-                raise ValueError(
-                    "if 'x' or 'y' are None, 'angle' must not be present"
-                )
+                raise ValueError("if 'x' or 'y' are None, 'angle' must not be present")
         if x is not None and y is not None and angle is None:
-            raise ValueError(
-                "if 'x' and 'y' are defined, 'angle' must be defined"
-            )
+            raise ValueError("if 'x' and 'y' are defined, 'angle' must be defined")
         if angle is not None and not (0 <= angle <= 360):
             raise ValueError("angle must be between 0 and 360")
