@@ -1,15 +1,50 @@
+from collections import namedtuple
+from collections.abc import Mapping, MutableMapping
+from copy import deepcopy
+from typing import Sequence, Union
+
 import attr
-
-try:
-    from collections.abc import Mapping, MutableMapping
-except ImportError:
-    from collections import Mapping, MutableMapping
-
+from fontTools.misc.transform import Transform
+from fontTools.pens.boundsPen import BoundsPen, ControlBoundsPen
 
 # sentinel value to signal a "lazy" object hasn't been loaded yet
 _NOT_LOADED = object()
 
 
+BoundingBox = namedtuple("BoundingBox", "xMin yMin xMax yMax")
+
+
+def getBounds(drawable, layer):
+    pen = BoundsPen(layer)
+    # raise 'KeyError' when a referenced component is missing from glyph set
+    pen.skipMissingComponents = False
+    drawable.draw(pen)
+    return pen.bounds if pen.bounds is None else BoundingBox(*pen.bounds)
+
+
+def getControlBounds(drawable, layer):
+    pen = ControlBoundsPen(layer)
+    # raise 'KeyError' when a referenced component is missing from glyph set
+    pen.skipMissingComponents = False
+    drawable.draw(pen)
+    return pen.bounds if pen.bounds is None else BoundingBox(*pen.bounds)
+
+
+def _deepcopy_unlazify_attrs(self, memo):
+
+    if getattr(self, "_lazy", True) and hasattr(self, "unlazify"):
+        self.unlazify()
+    return self.__class__(
+        **{
+            (a.name if a.name[0] != "_" else a.name[1:]): deepcopy(
+                getattr(self, a.name), memo
+            )
+            for a in attr.fields(self.__class__)
+            if a.init and a.metadata.get("copyable", True)
+        },
+    )
+
+
 @attr.s(slots=True, repr=False)
 class DataStore(MutableMapping):
     listdir = None
@@ -19,7 +54,7 @@ class DataStore(MutableMapping):
 
     _data = attr.ib(default=attr.Factory(dict), type=dict)
 
-    _reader = attr.ib(default=None, init=False, repr=False)
+    _reader = attr.ib(default=None, init=False, repr=False, eq=False)
     _scheduledForDeletion = attr.ib(default=attr.Factory(set), init=False, repr=False)
 
     @classmethod
@@ -34,6 +69,12 @@ class DataStore(MutableMapping):
             self._reader = reader
         return self
 
+    def unlazify(self):
+        for _ in self.items():
+            pass
+
+    __deepcopy__ = _deepcopy_unlazify_attrs
+
     # MutableMapping methods
 
     def __len__(self):
@@ -97,8 +138,10 @@ class DataStore(MutableMapping):
 
 
 class AttrDictMixin(Mapping):
-    """ Read attribute values using mapping interface. For use with Anchors and
-    Guidelines classes, where client code expects them to behave as dict.
+    """Read attribute values using mapping interface.
+
+    For use with Anchors and Guidelines classes, where client code
+    expects them to behave as dict.
     """
 
     def __getitem__(self, key):
@@ -114,3 +157,9 @@ class AttrDictMixin(Mapping):
 
     def __len__(self):
         return sum(1 for _ in self)
+
+
+def _convert_transform(t: Union[Transform, Sequence[Union[int, float]]]) -> Transform:
+    """Return a passed-in Transform as is, otherwise convert a sequence of
+    numbers to a Transform if need be."""
+    return t if isinstance(t, Transform) else Transform(*t)
